@@ -7,15 +7,18 @@
 #'   determines which values match this case. The right hand side (RHS)
 #'   provides the replacement value.
 #'
-#'   The LHS must evaluate to a logical vector. Each logical vector can
-#'   either have length 1 or a common length. All RHSs must evaluate to
-#'   the same type of vector.
+#'   The LHS must evaluate to a logical vector. The RHS does need to be
+#'   logical, but all RHSs must evaluate to the same type of vector.
 #'
-#'   These dots are evaluated with [explicit splicing][rlang::dots_list].
+#'   Both LHS and RHS may have the same length of either 1 or `n`. The
+#'   value of `n` must be consistent across all cases. The case of
+#'   `n == 0` is treated as a variant of `n != 1`.
+#'
+#'   These dots support [tidy dots][rlang::tidy-dots] features.
 #' @export
-#' @return A vector as long as the longest LHS, with the type (and
-#'   attributes) of the first RHS.  Inconsistent lengths or types will
-#'   generate an error.
+#' @return A vector of length 1 or `n`, matching the length of the logical
+#'   input or output vectors, with the type (and attributes) of the first
+#'   RHS. Inconsistent lengths or types will generate an error.
 #' @examples
 #' x <- 1:50
 #' case_when(
@@ -34,6 +37,31 @@
 #'   x %% 35 == 0 ~ "fizz buzz"
 #' )
 #'
+#' # All RHS values need to be of the same type. Inconsistent types will throw an error.
+#' # This applies also to NA values used in RHS: NA is logical, use
+#' # typed values like NA_real_, NA_complex, NA_character_, NA_integer_ as appropriate.
+#' case_when(
+#'   x %% 35 == 0 ~ NA_character_,
+#'   x %% 5 == 0 ~ "fizz",
+#'   x %% 7 == 0 ~ "buzz",
+#'   TRUE ~ as.character(x)
+#' )
+#' case_when(
+#'   x %% 35 == 0 ~ 35,
+#'   x %% 5 == 0 ~ 5,
+#'   x %% 7 == 0 ~ 7,
+#'   TRUE ~ NA_real_
+#' )
+#' # This throws an error as NA is logical not numeric
+#' \dontrun{
+#' case_when(
+#'   x %% 35 == 0 ~ 35,
+#'   x %% 5 == 0 ~ 5,
+#'   x %% 7 == 0 ~ 7,
+#'   TRUE ~ NA
+#' )
+#' }
+#'
 #' # case_when is particularly useful inside mutate when you want to
 #' # create a new variable that relies on a complex combination of existing
 #' # variables
@@ -49,14 +77,14 @@
 #'
 #' # Dots support splicing:
 #' patterns <- list(
-#'   TRUE ~ as.character(x),
-#'   x %%  5 == 0 ~ "fizz",
-#'   x %%  7 == 0 ~ "buzz",
-#'   x %% 35 == 0 ~ "fizz buzz"
+#'   x %% 35 == 0 ~ "fizz buzz",
+#'   x %% 5 == 0 ~ "fizz",
+#'   x %% 7 == 0 ~ "buzz",
+#'   TRUE ~ as.character(x)
 #' )
-#' case_when(!!! patterns)
+#' case_when(!!!patterns)
 case_when <- function(...) {
-  formulas <- dots_list(...)
+  formulas <- list2(...)
   n <- length(formulas)
 
   if (n == 0) {
@@ -85,22 +113,31 @@ case_when <- function(...) {
     value[[i]] <- eval_bare(f[[3]], env)
   }
 
-  m <- max(vapply(query, length, integer(1)))
+  lhs_lengths <- map_int(query, length)
+  rhs_lengths <- map_int(value, length)
+  all_lengths <- unique(c(lhs_lengths, rhs_lengths))
+  if (length(all_lengths) <= 1) {
+    m <- all_lengths[[1]]
+  } else {
+    non_atomic_lengths <- all_lengths[all_lengths != 1]
+    m <- non_atomic_lengths[[1]]
+    if (length(non_atomic_lengths) > 1) {
+      inconsistent_lengths <- non_atomic_lengths[-1]
+      lhs_problems <- lhs_lengths %in% inconsistent_lengths
+      rhs_problems <- rhs_lengths %in% inconsistent_lengths
+      problems <- lhs_problems | rhs_problems
+      bad_calls(
+        formulas[problems],
+        check_length_val(inconsistent_lengths, m, header = NULL, .abort = identity)
+      )
+    }
+  }
+
   out <- value[[1]][rep(NA_integer_, m)]
   replaced <- rep(FALSE, m)
 
   for (i in seq_len(n)) {
-    check_length(
-      query[[i]], out,
-      paste0("LHS of case ", i, " (", fmt_obj1(deparse_trunc(f_lhs(formulas[[i]]))), ")"),
-      "the longest input"
-    )
-
-    out <- replace_with(
-      out, query[[i]] & !replaced, value[[i]],
-      paste0("RHS of case ", i, " (", deparse_trunc(f_rhs(formulas[[i]])), ")"),
-      "the first output"
-    )
+    out <- replace_with(out, query[[i]] & !replaced, value[[i]], NULL)
     replaced <- replaced | (query[[i]] & !is.na(query[[i]]))
   }
 
