@@ -1,17 +1,19 @@
 #' Create a "src" object
 #'
-#' \code{src} is the standard constructor for srcs and \code{is.src} tests. 
+#' `src()` is the standard constructor for srcs and `is.src()` tests.
 #'
 #' @keywords internal
 #' @export
-#' @param subclass name of subclass. "src" is an abstract base class, so you 
-#'   must supply this value. \code{src_} is automatically prepended to the 
+#' @param subclass name of subclass. "src" is an abstract base class, so you
+#'   must supply this value. `src_` is automatically prepended to the
 #'   class name
-#' @param ... fields used by object
+#' @param ... fields used by object.
+#'
+#'   These dots are evaluated with [explicit splicing][rlang::dots_list].
 #' @param x object to test for "src"-ness.
 src <- function(subclass, ...) {
   subclass <- paste0("src_", subclass)
-  structure(list(...), class = c(subclass, "src"))
+  structure(dots_list(...), class = c(subclass, "src"))
 }
 
 #' @rdname src
@@ -29,14 +31,15 @@ print.src <- function(x, ...) {
 #' Most methods will not be documented because it's usually pretty obvious what
 #' possible results will be.
 #'
-#' @param x a data src. 
+#' @param x a data src.
 #' @export
+#' @keywords internal
 src_tbls <- function(x) {
   UseMethod("src_tbls")
 }
 
 #' Figure out if two sources are the same (or two tbl have the same source)
-#' 
+#'
 #' @param x,y src or tbls to test
 #' @return a logical flag
 #' @export
