@@ -11,16 +11,16 @@ tbl <- function(src, ...) {
 
 #' Create a "tbl" object
 #'
-#' \code{tbl} is the standard constructor for tbls. \code{as.tbl} coerces,
-#' and \code{is.tbl} tests.
+#' `tbl()` is the standard constructor for tbls. `as.tbl()` coerces,
+#' and `is.tbl()` tests.
 #'
 #' @keywords internal
 #' @export
 #' @param subclass name of subclass. "tbl" is an abstract base class, so you
-#'   must supply this value. \code{tbl_} is automatically prepended to the
+#'   must supply this value. `tbl_` is automatically prepended to the
 #'   class name
 #' @param object to test/coerce.
-#' @param ... For \code{tbl}, other fields used by class. For \code{as.tbl},
+#' @param ... For `tbl()`, other fields used by class. For `as.tbl()`,
 #'   other arguments passed to methods.
 #' @examples
 #' as.tbl(mtcars)
@@ -35,7 +35,7 @@ is.tbl <- function(x) inherits(x, "tbl")
 
 #' @export
 #' @rdname tbl
-#' @param x an object to coerce to a \code{tbl}
+#' @param x an object to coerce to a `tbl`
 as.tbl <- function(x, ...) UseMethod("as.tbl")
 
 #' @export
@@ -43,6 +43,19 @@ as.tbl.tbl <- function(x, ...) x
 
 #' List variables provided by a tbl.
 #'
+#' `tbl_vars()` returns all variables while `tbl_nongroup_vars()`
+#' returns only non-grouping variables.
+#'
 #' @export
 #' @param x A tbl object
-tbl_vars <- function(x) UseMethod("tbl_vars")
+#' @seealso [group_vars()] for a function that returns grouping
+#'   variables.
+#' @keywords internal
+tbl_vars <- function(x) {
+  UseMethod("tbl_vars")
+}
+#' @rdname tbl_vars
+#' @export
+tbl_nongroup_vars <- function(x) {
+  setdiff(tbl_vars(x), group_vars(x))
+}
