@@ -3,7 +3,7 @@
 
 #include <tools/scalar_type.h>
 #include <tools/utils.h>
-
+#include <dplyr/default_value.h>
 #include <dplyr/Result/Result.h>
 
 namespace dplyr {
@@ -16,7 +16,7 @@ public:
   Lead(SEXP data_, int n_, const RObject& def_, bool is_summary_) :
     data(data_),
     n(n_),
-    def(Vector<RTYPE>::get_na()),
+    def(default_value<RTYPE>()),
     is_summary(is_summary_)
   {
     if (!Rf_isNull(def_)) {
@@ -49,15 +49,6 @@ public:
     return out;
   }
 
-  virtual SEXP process(const FullDataFrame& df) {
-    int nrows = df.nrows();
-    Vector<RTYPE> out = no_init(nrows);
-    const SlicingIndex& index = df.get_index();
-    process_slice(out, index, index);
-    copy_most_attributes(out, data);
-    return out;
-  }
-
   virtual SEXP process(const SlicingIndex& index) {
     int nrows = index.size();
     Vector<RTYPE> out = no_init(nrows);
