@@ -135,7 +135,7 @@ public:
 };
 
 // powers both dense_rank and min_rank, see dplyr.cpp for how it is used
-template <int RTYPE, typename Increment, bool ascending = true>
+template <int RTYPE, typename Increment, bool ascending>
 class Rank_Impl : public Result, public Increment {
 public:
   typedef typename Increment::OutputVector OutputVector;
@@ -166,14 +166,6 @@ public:
     return IntegerVector(gdf.nrows(), 1);
   }
 
-  virtual SEXP process(const FullDataFrame& df) {
-    int n = df.nrows();
-    if (n == 0) return IntegerVector(0);
-    OutputVector out = no_init(n);
-    process_slice(out, df.get_index());
-    return out;
-  }
-
   virtual SEXP process(const SlicingIndex& index) {
     int n = index.size();
     if (n == 0) return IntegerVector(0);
@@ -186,7 +178,7 @@ private:
 
   void process_slice(OutputVector& out, const SlicingIndex& index) {
     map.clear();
-    Slice slice(data, index);
+    Slice slice(&data, index);
     int m = index.size();
     for (int j = 0; j < m; j++) {
       map[ slice[j] ].push_back(j);
@@ -242,7 +234,6 @@ public:
   RowNumber(SEXP data_) : data(data_) {}
 
   virtual SEXP process(const GroupedDataFrame& gdf) {
-    std::vector<int> tmp(gdf.max_group_size());
 
     int ng = gdf.ngroups();
     int n  = gdf.nrows();
@@ -251,16 +242,12 @@ public:
     IntegerVector out(n);
     for (int i = 0; i < ng; i++, ++git) {
       const SlicingIndex& index = *git;
+      Slice slice(&data, index);
+      Shield<SEXP> data_subset(slice);
+      OrderVisitors ordering_obj(data_subset, ascending);
+      IntegerVector tmp = ordering_obj.apply();
 
-      // tmp <- 0:(m-1)
       int m = index.size();
-      for (int j = 0; j < m; j++) tmp[j] = j;
-
-      Slice slice(data, index);
-      // order( gdf.group(i) )
-      Visitor visitor(slice);
-      Comparer comparer = Comparer(visitor);
-      std::sort(tmp.begin(), tmp.begin() + m, comparer);
       int j = m - 1;
       for (; j >= 0; j--) {
         if (Rcpp::traits::is_na<RTYPE>(slice[ tmp[j] ])) {
@@ -282,17 +269,14 @@ public:
     return IntegerVector(gdf.nrows(), 1);
   }
 
-  virtual SEXP process(const FullDataFrame& df) {
-    return process(df.get_index());
-  }
-
   virtual SEXP process(const SlicingIndex& index) {
     int nrows = index.size();
     if (nrows == 0) return IntegerVector(0);
-    IntegerVector x = seq(0, nrows - 1);
-    Slice slice(data, index);
-    Visitor visitor(slice);
-    std::sort(x.begin(), x.end(), Comparer(visitor));
+
+    Slice slice(&data, index);
+    Shield<SEXP> data_subset(slice);
+    OrderVisitors ordering_obj(data_subset, ascending);
+    IntegerVector x = ordering_obj.apply();
     IntegerVector out = no_init(nrows);
     int j = nrows - 1;
     for (; j >= 0; j--) {
@@ -309,7 +293,7 @@ public:
   }
 
 private:
-  SEXP data;
+  Vector<RTYPE> data;
 };
 
 template <int RTYPE, bool ascending = true>
@@ -324,8 +308,6 @@ public:
   Ntile(SEXP data_, double ntiles_) : data(data_), ntiles(ntiles_) {}
 
   virtual SEXP process(const GroupedDataFrame& gdf) {
-    std::vector<int> tmp(gdf.max_group_size());
-
     int ng = gdf.ngroups();
     int n  = gdf.nrows();
     if (n == 0) return IntegerVector(0);
@@ -334,15 +316,12 @@ public:
     for (int i = 0; i < ng; i++, ++git) {
       const SlicingIndex& index = *git;
 
-      // tmp <- 0:(m-1)
-      int m = index.size();
-      for (int j = 0; j < m; j++) tmp[j] = j;
-      Slice slice(data, index);
+      Slice slice(&data, index);
+      Shield<SEXP> data_subset(slice);
+      OrderVisitors ordering_obj(data_subset, ascending);
+      IntegerVector tmp = ordering_obj.apply();
 
-      // order( gdf.group(i) )
-      Visitor visitor(slice);
-      Comparer comparer = Comparer(visitor);
-      std::sort(tmp.begin(), tmp.begin() + m, comparer);
+      int m = index.size();
       int j = m - 1;
       for (; j >= 0; j--) {
         if (Rcpp::traits::is_na<RTYPE>(slice[tmp[j]])) {
@@ -364,17 +343,15 @@ public:
     return IntegerVector(gdf.nrows(), 1);
   }
 
-  virtual SEXP process(const FullDataFrame& df) {
-    return process(df.get_index());
-  }
-
   virtual SEXP process(const SlicingIndex& index) {
     int nrows = index.size();
     if (nrows == 0) return IntegerVector(0);
-    IntegerVector x = seq(0, nrows - 1);
-    Slice slice(data, index);
-    Visitor visitor(slice);
-    std::sort(x.begin(), x.end(), Comparer(visitor));
+
+    Slice slice(&data, index);
+    Shield<SEXP> data_subset(slice);
+    OrderVisitors ordering_obj(data_subset, ascending);
+    IntegerVector x = ordering_obj.apply();
+
     IntegerVector out = no_init(nrows);
     int i = nrows - 1;
     for (; i >= 0; i--) {
@@ -393,7 +370,7 @@ public:
   }
 
 private:
-  SEXP data;
+  Vector<RTYPE> data;
   double ntiles;
 };
 
@@ -418,12 +395,6 @@ public:
     return IntegerVector(gdf.nrows(), 1);
   }
 
-  virtual SEXP process(const FullDataFrame& df) {
-    if (df.nrows() == 0) return IntegerVector(0);
-    IntegerVector res = seq(1, df.nrows());
-    return res;
-  }
-
   virtual SEXP process(const SlicingIndex& index) {
     if (index.size() == 0) return IntegerVector(0);
     IntegerVector res = seq(1, index.size());
