@@ -8,6 +8,8 @@
 #include <dplyr/Result/GroupedCallProxy.h>
 #include <dplyr/Result/CallProxy.h>
 
+#include <dplyr/tbl_cpp.h>
+
 using namespace Rcpp;
 using namespace dplyr;
 
@@ -32,7 +34,7 @@ public:
     }
 
     if (n_neg > 0 && n_pos > 0) {
-      stop("Found %d positive indices and %d negative indices", n_pos, n_neg);
+      stop("Indices must be either all positive or all negative, not a mix of both. Found %d positive indices and %d negative indices", n_pos, n_neg);
     }
 
   }
@@ -83,7 +85,9 @@ DataFrame slice_grouped(GroupedDataFrame gdf, const QuosureList& dots) {
       // positive indexing
       int ntest = g_test.size();
       for (int j = 0; j < ntest; j++) {
-        if (!(g_test[j] > nr || g_test[j] == NA_INTEGER)) {
+        // only keep things inside inside 1:nr
+        // this skips 0 and NA (which is negative (-2^31) for INTSXP)
+        if (g_test[j] >= 1 && g_test[j] <= nr) {
           indx.push_back(indices[g_test[j] - 1]);
         }
       }
@@ -92,7 +96,7 @@ DataFrame slice_grouped(GroupedDataFrame gdf, const QuosureList& dots) {
       std::set<int> drop;
       int n = g_test.size();
       for (int j = 0; j < n; j++) {
-        if (g_test[j] != NA_INTEGER)
+        if (g_test[j] != NA_INTEGER && g_test[j] != 0)
           drop.insert(-g_test[j]);
       }
       int n_drop = drop.size();
@@ -124,6 +128,7 @@ DataFrame slice_grouped(GroupedDataFrame gdf, const QuosureList& dots) {
 
 DataFrame slice_not_grouped(const DataFrame& df, const QuosureList& dots) {
   CharacterVector names = df.names();
+  CharacterVector classes = Rf_getAttrib(df, R_ClassSymbol);
 
   const NamedQuosure& quosure = dots[0];
   Call call(quosure.expr());
@@ -143,23 +148,25 @@ DataFrame slice_not_grouped(const DataFrame& df, const QuosureList& dots) {
     std::vector<int> idx(n_pos);
     int j = 0;
     for (int i = 0; i < n_pos; i++) {
-      while (test[j] > nr || test[j] == NA_INTEGER) j++;
+      // skip until we are inside 1:nr
+      // this skips 0 and NA (which is negative (-2^31) for INTSXP)
+      while (test[j] > nr || test[j] < 1) j++;
       idx[i] = test[j++] - 1;
     }
 
-    return subset(df, idx, df.names(), classes_not_grouped());
+    return subset(df, idx, df.names(), classes);
   }
 
   // special case where only NA
   if (counter.get_n_negative() == 0) {
     std::vector<int> indices;
-    return subset(df, indices, df.names(), classes_not_grouped());
+    return subset(df, indices, df.names(), classes);
   }
 
   // just negatives (out of range is dealt with early in CountIndices).
   std::set<int> drop;
   for (int i = 0; i < n; i++) {
-    if (test[i] != NA_INTEGER)
+    if (test[i] != NA_INTEGER && test[i] != 0)
       drop.insert(-test[i]);
   }
   std::vector<int> indices;
@@ -179,7 +186,7 @@ DataFrame slice_not_grouped(const DataFrame& df, const QuosureList& dots) {
     indices.push_back(j);
   }
 
-  return subset(df, indices, df.names(), classes_not_grouped());
+  return subset(df, indices, df.names(), classes);
 }
 
 // [[Rcpp::export]]
