@@ -106,9 +106,9 @@ test_that("mutate recycles results of length 1", {
   list <- list(NULL)
 
   res <- mutate(group_by(df, x), int = int, str = str, num = num, bool = bool, list = list)
-  expect_equal(res$int , rep(int , 4))
-  expect_equal(res$str , rep(str , 4))
-  expect_equal(res$num , rep(num , 4))
+  expect_equal(res$int, rep(int, 4))
+  expect_equal(res$str, rep(str, 4))
+  expect_equal(res$num, rep(num, 4))
   expect_equal(res$bool, rep(bool, 4))
   expect_equal(res$list, rep(list, 4))
 })
@@ -117,7 +117,7 @@ test_that("mutate recycles results of length 1", {
 test_that("mutate handles out of data variables", {
   today <- Sys.Date()
   now <- Sys.time()
-  df  <- data.frame(x = c(2, 2, 3, 3))
+  df <- data.frame(x = c(2, 2, 3, 3))
   gdf <- group_by(df, x)
 
   int  <- c(1L, 2L)
@@ -131,12 +131,12 @@ test_that("mutate handles out of data variables", {
     gdf,
     int = int, str = str, num = num, bool = bool, dat = dat, tim = tim
   )
-  expect_equal(res$int , rep(int , 2))
-  expect_equal(res$str , rep(str , 2))
-  expect_equal(res$num , rep(num , 2))
+  expect_equal(res$int, rep(int, 2))
+  expect_equal(res$str, rep(str, 2))
+  expect_equal(res$num, rep(num, 2))
   expect_equal(res$bool, rep(bool, 2))
-  expect_equal(res$dat , rep(dat , 2))
-  expect_equal(res$tim , rep(tim , 2))
+  expect_equal(res$dat, rep(dat, 2))
+  expect_equal(res$tim, rep(tim, 2))
 
   int <- 1:6
   expect_error(
@@ -161,12 +161,12 @@ test_that("mutate handles out of data variables", {
     tbl_df(df),
     int = int, str = str, num = num, bool = bool, tim = tim, dat = dat
   )
-  expect_equal(res$int , int)
-  expect_equal(res$str , str)
-  expect_equal(res$num , num)
+  expect_equal(res$int, int)
+  expect_equal(res$str, str)
+  expect_equal(res$num, num)
   expect_equal(res$bool, bool)
-  expect_equal(res$dat , dat)
-  expect_equal(res$tim , tim)
+  expect_equal(res$dat, dat)
+  expect_equal(res$tim, tim)
 })
 
 test_that("mutate handles passing ...", {
@@ -204,7 +204,6 @@ test_that("mutate handles passing ...", {
   expect_equal(res$x2, rep(2, 4))
   expect_equal(res$before, rep("before", 4))
   expect_equal(res$after, rep("after", 4))
-
 })
 
 test_that("mutate fails on unsupported column type", {
@@ -241,10 +240,15 @@ test_that("mutate errors when results are not compatible accross groups (#299)",
 })
 
 test_that("assignments don't overwrite variables (#315)", {
-   expect_equal(
-     mutate(mtcars, cyl2 = { mpg <- cyl ^ 2; -mpg }),
-     mutate(mtcars, cyl2 = -cyl ^ 2)
-   )
+  expect_equal(
+    mutate(mtcars,
+      cyl2 = {
+        mpg <- cyl^2
+        -mpg
+      }
+    ),
+    mutate(mtcars, cyl2 = -cyl^2)
+  )
 })
 
 test_that("hybrid evaluator uses correct environment (#403)", {
@@ -268,7 +272,7 @@ test_that("mutate strips names, but only if grouped (#1689, #2675)", {
   data <- data_frame(a = 1:3) %>% mutate(b = setNames(nm = a))
   expect_equal(names(data$b), as.character(1:3))
 
-  data <- data_frame(a = 1:3) %>% rowwise %>% mutate(b = setNames(nm = a))
+  data <- data_frame(a = 1:3) %>% rowwise() %>% mutate(b = setNames(nm = a))
   expect_null(names(data$b))
 
   data <- data_frame(a = c(1, 1, 2)) %>% group_by(a) %>% mutate(b = setNames(nm = a))
@@ -307,10 +311,10 @@ test_that("mutate gives a nice error message if an expression evaluates to NULL
 
 test_that("mutate(rowwise_df) makes a rowwise_df (#463)", {
   one_mod <- data.frame(grp = "a", x = runif(5, 0, 1)) %>%
-    tbl_df %>%
+    tbl_df() %>%
     mutate(y = rnorm(x, x * 2, 1)) %>%
     group_by(grp) %>%
-    do(mod = lm(y~x, data = .))
+    do(mod = lm(y ~ x, data = .))
 
   out <- one_mod %>%
     mutate(rsq = summary(mod)$r.squared) %>%
@@ -323,7 +327,7 @@ test_that("mutate(rowwise_df) makes a rowwise_df (#463)", {
 })
 
 test_that("mutate allows list columns (#555)", {
-  df  <- data.frame(x = c("a;b", "c;d;e"), stringsAsFactors = FALSE)
+  df <- data.frame(x = c("a;b", "c;d;e"), stringsAsFactors = FALSE)
   res <- mutate(df, pieces = strsplit(x, ";"))
   expect_equal(res$pieces, list(c("a", "b"), c("c", "d", "e")))
 })
@@ -360,8 +364,8 @@ test_that("hybrid not get in the way of order_by (#169)", {
 
 test_that("mutate supports difftime objects (#390)", {
   df <- data_frame(
-    grp =   c(1, 1,  2, 2),
-    val =   c(1, 3,  4, 6),
+    grp = c(1, 1, 2, 2),
+    val = c(1, 3, 4, 6),
     date1 = c(rep(Sys.Date() - 10, 2), rep(Sys.Date() - 20, 2)),
     date2 = Sys.Date() + c(1, 2, 1, 2),
     diffdate = difftime(date2, date1, unit = "days")
@@ -391,10 +395,12 @@ test_that("mutate works on zero-row grouped data frame (#596)", {
 })
 
 test_that("Non-ascii column names in version 0.3 are not duplicated (#636)", {
+  # Currently failing (#2967)
+  skip_on_os("windows")
   df <- data_frame(a = "1", b = "2")
   names(df) <- c("a", enc2native("\u4e2d"))
 
-  res <- df %>% mutate_all(funs(as.numeric)) %>% names
+  res <- df %>% mutate_all(funs(as.numeric)) %>% names()
   expect_equal(res, names(df))
 })
 
@@ -428,7 +434,6 @@ test_that("mutate forbids POSIXlt results (#670)", {
     "Column `time` is of unsupported class POSIXlt",
     fixed = TRUE
   )
-
 })
 
 test_that("constant factor can be handled by mutate (#715)", {
@@ -450,8 +455,8 @@ test_that("row_number handles empty data frames (#762)", {
   )
   expect_equal(
     names(res),
-     c("a", "row_number_0", "row_number_a", "ntile", "min_rank", "percent_rank", "dense_rank", "cume_dist")
-   )
+    c("a", "row_number_0", "row_number_a", "ntile", "min_rank", "percent_rank", "dense_rank", "cume_dist")
+  )
   expect_equal(nrow(res), 0L)
 })
 
@@ -463,7 +468,7 @@ test_that("no utf8 invasion (#722)", {
 
 test_that("mutate works on empty data frames (#1142)", {
   df <- data.frame()
-  res <- df %>% mutate
+  res <- df %>% mutate()
   expect_equal(nrow(res), 0L)
   expect_equal(length(res), 0L)
 
@@ -486,16 +491,23 @@ test_that("mutate handles 0 rows rowwise (#1300)", {
 
   res <- f()
   expect_equal(nrow(res), 0L)
+})
+
+test_that("rhs of mutate cannot be a data frame (#3298)", {
+  df <- data.frame("a" = c(1, 2, 3), "b" = c(2, 3, 4), "base_col" = c(3, 4, 5))
+  expect_error(
+    mutate(df, new_col = data.frame(1:3)),
+    "Column `new_col` is of unsupported class data.frame"
+  )
 
   expect_error(
-    a %>% mutate(b = f()),
-    "Column `b` must be length 1 (the number of rows), not 2",
-    fixed = TRUE
+    mutate(group_by(df, a), new_col = data.frame(1:3)),
+    "Column `new_col` is of unsupported class data.frame"
   )
+
   expect_error(
-    a %>% rowwise() %>% mutate(b = f()),
-    "Column `b` must be length 1 (the group size), not 2",
-    fixed = TRUE
+    mutate(rowwise(df), new_col = data.frame(1:3)),
+    "Column `new_col` is of unsupported class data.frame"
   )
 })
 
@@ -584,7 +596,7 @@ test_that("mutate disambiguates NA and NaN (#1448)", {
   expect_true(is.nan(res$pass2[1]))
 
   res <- Pass %>%
-    rowwise %>%
+    rowwise() %>%
     mutate(pass2 = P2 / (P2 + F2))
   expect_true(is.nan(res$pass2[1]))
 
@@ -648,7 +660,7 @@ test_that("grouped mutate coerces integer + double -> double (#1892)", {
 })
 
 test_that("grouped mutate coerces factor + character -> character (WARN) (#1892)", {
-  factor_or_character <- function(x)  {
+  factor_or_character <- function(x) {
     if (x > 3) {
       return(factor("hello"))
     } else {
@@ -685,7 +697,7 @@ test_that("Adding a Column of NA to a Grouped Table gives expected results (#164
 
 test_that("Deep copies are performed when needed (#1463)", {
   res <- data.frame(prob = c(F, T)) %>%
-    rowwise %>%
+    rowwise() %>%
     mutate(model = list(x = prob))
   expect_equal(unlist(res$model), c(FALSE, TRUE))
 
@@ -708,40 +720,45 @@ test_that("mutate() names pronouns correctly (#2686)", {
 
 test_that("mutate() supports unquoted values", {
   df <- tibble(g = c(1, 1, 2, 2, 2), x = 1:5)
-  expect_identical(mutate(df, out = !! 1), mutate(df, out = 1))
-  expect_identical(mutate(df, out = !! 1:5), mutate(df, out = 1:5))
-  expect_identical(mutate(df, out = !! quote(1:5)), mutate(df, out = 1:5))
-  expect_error(mutate(df, out = !! 1:2), "must be length 5 (the number of rows)", fixed = TRUE)
-  expect_error(mutate(df, out = !! get_env()), "unsupported type")
+  expect_identical(mutate(df, out = !!1), mutate(df, out = 1))
+  expect_identical(mutate(df, out = !!(1:5)), mutate(df, out = 1:5))
+  expect_identical(mutate(df, out = !!quote(1:5)), mutate(df, out = 1:5))
+  expect_error(mutate(df, out = !!(1:2)), "must be length 5 (the number of rows)", fixed = TRUE)
+  expect_error(mutate(df, out = !!get_env()), "unsupported type")
 
   gdf <- group_by(df, g)
-  expect_identical(mutate(gdf, out = !! 1), mutate(gdf, out = 1))
-  expect_identical(mutate(gdf, out = !! 1:5), group_by(mutate(df, out = 1:5), g))
-  expect_error(mutate(gdf, out = !! quote(1:5)), "must be length 2 (the group size)", fixed = TRUE)
-  expect_error(mutate(gdf, out = !! 1:2), "must be length 5 (the number of rows)", fixed = TRUE)
-  expect_error(mutate(gdf, out = !! get_env()), "unsupported type")
+  expect_identical(mutate(gdf, out = !!1), mutate(gdf, out = 1))
+  expect_identical(mutate(gdf, out = !!(1:5)), group_by(mutate(df, out = 1:5), g))
+  expect_error(mutate(gdf, out = !!quote(1:5)), "must be length 2 (the group size)", fixed = TRUE)
+  expect_error(mutate(gdf, out = !!(1:2)), "must be length 5 (the number of rows)", fixed = TRUE)
+  expect_error(mutate(gdf, out = !!get_env()), "unsupported type")
 })
 
+test_that("gathering handles promotion from raw", {
+  df <- data_frame(a = 1:4, g = c(1, 1, 2, 2))
+  # collecting raw in the first group, then other types
+  expect_identical(
+    df %>% group_by(g) %>% mutate(b = if (all(a < 3)) as.raw(a) else a) %>% pull(b),
+    1:4
+  )
+  expect_identical(
+    df %>% group_by(g) %>% mutate(b = if (all(a < 3)) as.raw(a) else as.numeric(a)) %>% pull(b),
+    as.numeric(1:4)
+  )
+})
 
 # Error messages ----------------------------------------------------------
 
-test_that("mutate fails gracefully on non-vector columns (#1803)", {
+test_that("mutate handles raw vectors in columns (#1803)", {
   df <- data_frame(a = 1:3, b = as.raw(1:3))
-  expect_error(
-    mutate(df, a = 1),
-    "Column `b` is of unsupported type raw vector",
-    fixed = TRUE
-  )
-  expect_error(
-    mutate(df, b = 1),
-    "Column `b` is of unsupported type raw vector",
-    fixed = TRUE
-  )
-  expect_error(
-    mutate(df, c = 1),
-    "Column `b` is of unsupported type raw vector",
-    fixed = TRUE
-  )
+  expect_identical(mutate(df, a = 1), data_frame(a = 1, b = as.raw(1:3)))
+  expect_identical(mutate(df, b = 1), data_frame(a = 1:3, b = 1))
+  expect_identical(mutate(df, c = 1), data_frame(a = 1:3, b = as.raw(1:3), c = 1))
+  expect_identical(mutate(df, c = as.raw(a)), data_frame(a = 1:3, b = as.raw(1:3), c = as.raw(1:3)))
+
+  df <- data_frame(a = 1:4, g = c(1, 1, 2, 2))
+  expect_identical(mutate(df, b = as.raw(a)) %>% group_by(g) %>% pull(b), as.raw(1:4))
+  expect_identical(mutate(df, b = as.raw(a)) %>% rowwise() %>% pull(b), as.raw(1:4))
 })
 
 test_that("grouped mutate errors on incompatible column type (#1641)", {
@@ -763,3 +780,21 @@ test_that("can reuse new variables", {
     data.frame(c = 1, gc = 1)
   )
 })
+
+test_that("can use character vectors in grouped mutate (#2971)", {
+  df <-
+    data_frame(x = 1:10000) %>%
+    group_by(x) %>%
+    mutate(
+      y = as.character(runif(1L)),
+      z = as.character(runif(1L))
+    )
+
+  expect_error(df %>% distinct(x, .keep_all = TRUE), NA)
+})
+
+test_that("mutate() to UTF-8 column names", {
+  df <- data_frame(a = 1) %>% mutate("\u5e78" := a)
+
+  expect_equal(colnames(df), c("a", "\u5e78"))
+})
